//
// Copyright (C) 1996, 2008  Control Technology Corporation.  All Rights Reserved.
//
//  MODULE:   ctccom32v2.h
//
//  PURPOSE:
//    Contains declarations for all globally scoped names in the DLL.
//

#ifndef INC_CTCCOM32_H
#define INC_CTCCOM32_H		1

#define DLLEXPORT __declspec(dllexport)

#ifndef MAXMSGSIZE
// This covers worst case CTC message reply
#define MAXMSGSIZE 256	// KRH bigger for Emcore packet
#define MAXCOMMSGSIZE (1492-8)	// KRH bigger for QS4 packet
#endif

// function return values
#define FAILURE							0
#define SUCCESS							1
#define ERROR_UNKNOWN					255 
#define ERROR_WIN_TIMER					246 
#define ERROR_NET_TIMEOUT				236 
#define ERROR_NET_NOTREADY				235 
#define ERROR_BAD_CONNECTION_COUNT		234
#define ERROR_CT_RANGE					216 
#define ERROR_BAD_CHECKSUM				215 
#define ERROR_BAD_RESPONSE				206 
#define ERROR_NOT_CT_PROTOCOL			205
#define ERROR_UNSUPP_REMOTE_OPER		196 
#define ERROR_UNKNOWN_REM_NODE			195 
#define ERROR_REMOTE_ACCESS				194 
#define ERROR_CT_FILE_NOT_FOUND			187
#define ERROR_OBJECT_FILE_NOT_FOUND		186
#define ERROR_FILE_WRONG_ARCH			185
#define ERROR_FILE_NOT_OBJECT			184
#define ERROR_BAD_OBJECT_RECORD			183
#define ERROR_BAD_OBJECT_LENGTH			182
#define ERROR_BAD_POSITION				181
#define ERROR_EOF						180
#define ERROR_BAD_PORT					179
#define ERROR_NET_IN_USE				178
#define ERROR_SERIAL_TIMEOUT			170
#define ERROR_WARNING					169
#define ERROR_ILLEGAL_BAUD_RATE			168
#define ERROR_ILLEGAL_PARITY_SETTING	167
#define ERROR_ILLEGAL_WORD_LENGTH		166
#define ERROR_ILLEGAL_STOP_BITS			165
#define ERROR_ERROR						160
#define ERROR_PORT_NOT_FOUND			159
#define ERROR_PORT_IN_USE				158

// binary protocol format constants
#define BINARY_PROTOCOL    ((BYTE) 0x01)
#define END_OF_PACKET      ((BYTE) 0xFF)

// binary protocol message types
#define MSG_LOAD_PROGRAM_PACKET			((BYTE) 0)
#define MSG_ENTER_PROGRAM_MODE			((BYTE) 1)
#define MSG_LEAVE_PROGRAM_MODE			((BYTE) 2)
#define MSG_UNLOAD_PROGRAM_PACKET		((BYTE) 3)
#define MSG_PROGRAM_PACKET				((BYTE) 4)
#define MSG_GET_ID_CODES				((BYTE) 5)
#define MSG_GOT_ID_CODES				((BYTE) 6)
#define MSG_OLD_GET_STATUS				((BYTE) 7)
#define MSG_OLD_GOT_STATUS				((BYTE) 8)
#define MSG_GET_REGISTER				((BYTE) 9)
#define MSG_GOT_REGISTER				((BYTE) 10)
#define MSG_SET_REGISTER				((BYTE) 11)
#define MSG_12							((BYTE) 12)
#define MSG_GET_IO_COUNTS				((BYTE) 13)
#define MSG_GOT_IO_COUNTS				((BYTE) 14)
#define MSG_GET_INPUTS					((BYTE) 15)
#define MSG_GOT_INPUTS					((BYTE) 16)
#define MSG_GET_FLAG					((BYTE) 17)
#define MSG_GOT_FLAG					((BYTE) 18)
#define MSG_SET_FLAG					((BYTE) 19)
#define MSG_20							((BYTE) 20)
#define MSG_GET_OUTPUTS					((BYTE) 21)
#define MSG_GOT_OUTPUTS					((BYTE) 22)
#define MSG_GET_SERVO_POSITION			((BYTE) 23)
#define MSG_GOT_SERVO_POSITION			((BYTE) 24)
#define MSG_SET_OUTPUTS					((BYTE) 25)
#define MSG_26							((BYTE) 26)
#define MSG_GET_SERVO_INPUT				((BYTE) 27)
#define MSG_GOT_SERVO_INPUT				((BYTE) 28)
#define MSG_GET_ANALOG_INPUT			((BYTE) 29)
#define MSG_GOT_ANALOG_INPUT			((BYTE) 30)
#define MSG_GET_ANALOG_OUTPUT			((BYTE) 31)
#define MSG_GOT_ANALOG_OUTPUT			((BYTE) 32)
#define MSG_SET_ANALOG_OUTPUT			((BYTE) 33)
#define MSG_34							((BYTE) 34)
#define MSG_GET_STATUS					((BYTE) 35)
#define MSG_GOT_STATUS_1of4				((BYTE) 36)
#define MSG_GOT_STATUS_2of4				((BYTE) 37)
#define MSG_GOT_STATUS_3of4				((BYTE) 38)
#define MSG_GOT_STATUS_4of4				((BYTE) 39)
#define MSG_SET_EA_OUTPUT				((BYTE) 40)
#define MSG_LOAD_EA_PROGRAM_PACKET		((BYTE) 41)
#define MSG_42							((BYTE) 42)
#define MSG_UNLOAD_EA_PROGRAM_PACKET	((BYTE) 43)
#define MSG_EA_PROGRAM_PACKET			((BYTE) 44)
#define MSG_DUMP_USER_MEMORY			((BYTE) 45)
#define MSG_USER_MEMORY					((BYTE) 46)
#define MSG_GET_SERVO_ERROR				((BYTE) 47)
#define MSG_GOT_SERVO_ERROR				((BYTE) 48)
#define MSG_GET_DATA_TABLE_SIZE			((BYTE) 49)
#define MSG_GOT_DATA_TABLE_SIZE			((BYTE) 50)
#define MSG_SET_DATA_TABLE_SIZE			((BYTE) 51)
#define MSG_52							((BYTE) 52)
#define MSG_GET_DATA_TABLE_ELEMENT		((BYTE) 53)
#define MSG_GOT_DATA_TABLE_ELEMENT		((BYTE) 54)
#define MSG_SET_DATA_TABLE_ELEMENT		((BYTE) 55)
#define MSG_56							((BYTE) 56)
#define MSG_GET_DATA_TABLE_ROW			((BYTE) 57)
#define MSG_GOT_DATA_TABLE_ROW			((BYTE) 58)
#define MSG_SET_DATA_TABLE_ROW			((BYTE) 59)
#define MSG_60							((BYTE) 60)
#define MSG_GET_CONTROLLER_STATE		((BYTE) 61)
#define MSG_GOT_CONTROLLER_STATE		((BYTE) 62)
#define MSG_SET_CONTROLLER_STATE		((BYTE) 63)
#define MSG_64							((BYTE) 64)
#define MSG_GET_SYSCONFIG_BYTE			((BYTE) 65)
#define MSG_GOT_SYSCONFIG_BYTE			((BYTE) 66)
#define MSG_SET_SYSCONFIG_BYTE			((BYTE) 67)
#define MSG_68							((BYTE) 68)
#define MSG_GET_OTHER_IO_COUNTS			((BYTE) 69)
#define MSG_GOT_OTHER_IO_COUNTS			((BYTE) 70)
#define MSG_GET_32_ANALOG_INS			((BYTE) 71)
#define MSG_GOT_32_ANALOG_INS			((BYTE) 72)
#define MSG_GET_32_ANALOG_OUTS			((BYTE) 73)
#define MSG_GOT_32_ANALOG_OUTS			((BYTE) 74)
#define MSG_GET_50_REGISTERS			((BYTE) 75)
#define MSG_GOT_50_REGISTERS			((BYTE) 76)
#define MSG_GET_16_REGISTERS			((BYTE) 77)
#define MSG_GOT_16_REGISTERS			((BYTE) 78)
#define MSG_GET_128_INPUTS				((BYTE) 79)
#define MSG_GOT_128_INPUTS				((BYTE) 80)
#define MSG_GET_128_OUTPUTS				((BYTE) 81)
#define MSG_GOT_128_OUTPUTS				((BYTE) 82)
#define MSG_SET_64_ANALOG_OUTS			((BYTE) 85)
#define MSG_GET_N_REGISTERS				((BYTE) 87)  // new enhancement KRH 1/2006
#define MSG_GOT_N_REGISTERS				((BYTE) 88)  // new enhancement KRH 1/2006

// special message for 2217 v3.8 data structure 
#define MSG_GET_2217_DATA				((BYTE) 83)
#define MSG_GOT_2217_DATA				((BYTE) 84)

// Variant data types
#define MSG_GET_VREGISTERROW			((BYTE) 89)
#define MSG_GOT_VREGISTERROW			((BYTE) 90)
#define MSG_GET_VPROPERTIES				((BYTE) 91)
#define MSG_GOT_VPROPERTIES				((BYTE) 92)
#define MSG_GET_VREGISTER				((BYTE) 93)
#define MSG_GOT_VREGISTER				((BYTE) 94)
#define MSG_SET_VREGISTER				((BYTE) 95)
#define MSG_GET_RUNCOMMAND				((BYTE) 97)
#define MSG_GOT_RUNCOMMAND				((BYTE) 98)


// devicenet and/or distributed io messages
#define MSG_UNLOAD_REMOTE_DATA			((BYTE) 101)
#define MSG_REMOTE_DATA_PACKET			((BYTE) 102)
#define MSG_LOAD_REMOTE_DATA			((BYTE) 103)
#define MSG_104							((BYTE) 104)
#define MSG_GET_REMOTE_IO				((BYTE) 105)
#define MSG_GOT_REMOTE_IO				((BYTE) 106)
#define MSG_GET_VREGISTER_BLOCK			((BYTE) 109)
#define MSG_GOT_VREGISTER_BLOCK			((BYTE) 110)
#define MSG_SET_VREGISTER_BLOCK			((BYTE) 111)

#define MSG_GET_VREGISTER_RANDOM_BLOCK	((BYTE) 113)
#define MSG_GOT_VREGISTER_RANDOM_BLOCK	((BYTE) 114)


// some useful things
const int NO_PARITY	= 0;
const int ODD_PARITY = 1;
const int EVEN_PARITY = 2;
const int MARK_PARITY = 3;
const int SPACE_PARITY = 4;
const int FOUR_DATA_BITS = 4;
const int FIVE_DATA_BITS = 5;
const int SIX_DATA_BITS = 6;
const int SEVEN_DATA_BITS = 7;
const int EIGHT_DATA_BITS = 8;
const int ONE_STOP_BITS = 0;
const int ONE_FIVE_STOP_BITS = 1;
const int TWO_STOP_BITS	= 2;

// data types for ctc binary protocol responses
typedef LONG		DIGS_128[4];
typedef LONG		ANGS_32[32];
typedef LONG		REGS_16[16];
typedef LONG		REGS_50[50];

// program file statistics
typedef struct  {
	LONG	lSteps;		// Number of steps
  	LONG	lBytes;		// Number of bytes
  	LONG	lErrors;		// Number of compilation errors
  	LONG	lWarnings;	// Number of compilation warnings
} CT_PROG_COUNT;

// data table row structure 
const int CT_MAX_COLS = 255;
typedef struct
{
	LONG	lRow;
	LONG	lColumns[CT_MAX_COLS];
} CT_DT_ROW;

// io count structure 
typedef struct {
	LONG	lFlags;
	LONG	lDigitalInputs;		
	LONG	lDigitalOutputs;
	LONG	lSteppingMotors;
	LONG	lServoMotors;
	LONG	lAnalogInputs;
	LONG	lAnalogOutputs;
} CT_IO_COUNT;

// misc io count structure 
typedef struct {
	LONG	lPrototypes;
	LONG	lHighSpeedCounters;
	LONG	lThumbwheels;
	LONG	lDisplays;
} CT_MISC_IO_COUNT;

// step status bank structure
typedef struct {
	LONG	lStopped;
	LONG	lFaultType;
	LONG	lFaultStep;		
	LONG	lFaultData;
	LONG	lTaskStep[8];
	LONG	lTaskMask1[8];
	LONG	lTaskMask2[8];
} CT_TASK_BANK;

// connection structure
typedef struct {
	LONG	lConnType;
	LONG	lSrcAddr;
	LONG	lDestAddr;
	LONG	lTimeout;
	BOOL	fConnected;
	BYTE	srcEAddr[6];
	BYTE	destEAddr[6];
} CT_CONN_INFO;

// message structure
typedef struct {
	LONG	lMsglen;
	BYTE	sMsgbuf[ MAXMSGSIZE];	// packet buffer  (CTLIB style packet)
	LONG	lRsplen;
	BYTE	sRspbuf[ 1536];	// packet buffer  (CTLIB style packet)
} CT_MSG_INFO;

// error structure
typedef struct {
	LONG	lErrCode;
	BYTE	sErrBuf[ MAXMSGSIZE];
} CT_ERR_INFO;

// Universal variant storage structure

#define BIT0 0x0001
#define BIT1 0x0002
#define BIT2 0x0004
#define BIT3 0x0008
#define BIT4 0x0010

#define VARIANT_MAX_STRING 223
#define VARIANT_INTEGER BIT0
#define VARIANT_UINTEGER BIT1
#define VARIANT_STRING BIT2
#define VARIANT_FLOAT BIT3
#define VARIANT_DOUBLE BIT4

typedef struct
{
	int numAccess;	// number of items to access
	int rowInc;		// row increment, if 0 just read columns based upon colInc.
	int colInc;		// col increment, if 0 just increment rows.
	int arraysizeCols;	// Used on write operation, -1 do not expand existing columns, else columns desired.  Rows will automatically grow as needed
} BLOCKACCESS;

typedef struct
{
	int type;					// type of storage being used or requested
								// If -1 on read then return current, else set to type want.
								// On write must set to type that is stored within this structure
	unsigned char precision;  	// double to string conversion precision %.6f default
								// On read is what is presently set, write what want. 
	unsigned char flags;		// special flags for processing so far only
								// VARIANT_INDIRECTION_FLAG used, can be used to set property
								// in ->settings on write operation, no effect on read.  Written
								// value becomes register to reference for further operations.
	unsigned char cmd;			// 00, no operation other than read/write specified, else do defined
								// operation.  Currently have write for properties access to 'settings'
								// VARIANT_CMD_SET_INDIRECTION and VARIANT_CMD_CLEAR_INDIRECTION, 
								// write value ignored.
	unsigned char pad;
    unsigned short taskHandle; // task number (offset in task array + 1, where 0 is 1) or handle thus usable from remote or C API, 4096 to 65535, set to 0 for public reg.
    unsigned short slength;    // this is reserved for later use and possible string length if want unsigned char, 0  255 values, VARIANT_BYTE, future type
	unsigned int indexCol;	  // Column dimension index reference
	unsigned int indexRow;	  // Row dimension index reference
	union						// Data that was read or has been written of 'type'
	{
		int iValue;
		unsigned int uiValue;
		float fValue;
		double dValue;
		unsigned int dSwap[2];	// used to swap doubles for PC access
		char sValue[VARIANT_MAX_STRING+1];
	} data;
} VARIANT_STORAGE;

#define MAX_VARIANT_BLOCK_32BITS	346		// 346 integers
#define MAX_VARIANT_BLOCK_64BITS 	173 	// 173 doubles
#define MAX_VARIANT_RANDOM_BLOCK 	(MAX_VARIANT_BLOCK_32BITS/3) 	// 115 items

#define MAX_VARIANT_BLOCK_32BITS_SERIAL 50
#define MAX_VARIANT_BLOCK_64BITS_SERIAL 	(MAX_VARIANT_BLOCK_32BITS_SERIAL/2) 	// 25 doubles
#define MAX_VARIANT_RANDOM_BLOCK_SERIAL 	(MAX_VARIANT_BLOCK_32BITS_SERIAL/3) 	// 16 items


#if 1
typedef struct
{
	int reg;  // may at some point reserve the upper 16 bits of this integer for 'type' req.
	int row;
	int col;
} VARIANT_ACCESS_REQUEST;
typedef struct
{
	long reg;  // may at some point reserve the upper 16 bits of this integer for 'type' req.
	long row;
	long col;
} VARIANT_ACCESS_REQUEST_VB;

#endif

// Allow for block reads
typedef struct
{
	int type;					// type of storage being used or requested
								// If -1 on read then return current, else set to type want.
								// On write must set to type that is stored within this structure
								// write not supported for block access
								// If block access type field will be 0 if error else type of first cell.
								// slength will be the number of elements returned within data.block.?[n]
	unsigned char precision;  	// double to string conversion precision %.6f default
								// On read is what is presently set, write what want. 
	unsigned char flags;		// special flags for processing so far only
								// VARIANT_INDIRECTION_FLAG used, can be used to set property
								// in ->settings on write operation, no effect on read.  Written
								// value becomes register to reference for further operations.
	unsigned char cmd;			// 00, no operation other than read/write specified, else do defined
								// operation.  Currently have write for properties access to 'settings'
								// VARIANT_CMD_SET_INDIRECTION and VARIANT_CMD_CLEAR_INDIRECTION, 
								// write value ignored.
	unsigned char pad;
    unsigned short taskHandle; // task number (offset in task array + 1, where 0 is 1) or handle thus usable from remote or C API, 4096 to 65535, set to 0 for public reg.
    unsigned short slength;    // this is reserved for later use and possible string length if want unsigned char, 0  255 values, VARIANT_BYTE, future type
	unsigned int indexCol;	  // Column dimension index reference
	unsigned int indexRow;	  // Row dimension index reference
	union						// Data that was read or has been written of 'type'
	{
		int iValue;
		unsigned int uiValue;
		float fValue;
		double dValue;
		unsigned int dSwap[2];	// used to swap doubles for PC access
		char *psValue;
		char sValue[VARIANT_MAX_STRING+1];
								// will be stored in same VARIANT_STORAGE upon return, thus data.blockread.numAccess * sizeof(variant type)
		// if BLOCKACCESS then iValue[n], fValue[n], or dValue[n] up to MAX_VARIANT_READBLOCK_SIZE
		struct
		{
			BLOCKACCESS blockaccess;	// Defines block read of variant cells, data storage must be big enough since
			union	  
			{
				int ibValue[MAX_VARIANT_BLOCK_32BITS];	   
				float fbValue[MAX_VARIANT_BLOCK_32BITS];   
				double dbValue[MAX_VARIANT_BLOCK_64BITS];  
				union
				{
					int ibValue;	   
					float fbValue;   
					double dbValue;  
				} random[MAX_VARIANT_RANDOM_BLOCK];
				VARIANT_ACCESS_REQUEST request[MAX_VARIANT_RANDOM_BLOCK];
			};
		} block;
	} data;
} VARIANT_STORAGE_BLOCK;

typedef struct
{
	int type;					// type of storage being used or requested
								// If -1 on read then return current, else set to type want.
								// On write must set to type that is stored within this structure
								// write not supported for block access
								// If block access type field will be 0 if error else type of first cell.
								// slength will be the number of elements returned within data.block.?[n]
	unsigned char precision;  	// double to string conversion precision %.6f default
								// On read is what is presently set, write what want. 
	unsigned char flags;		// special flags for processing so far only
								// VARIANT_INDIRECTION_FLAG used, can be used to set property
								// in ->settings on write operation, no effect on read.  Written
								// value becomes register to reference for further operations.
	unsigned char cmd;			// 00, no operation other than read/write specified, else do defined
								// operation.  Currently have write for properties access to 'settings'
								// VARIANT_CMD_SET_INDIRECTION and VARIANT_CMD_CLEAR_INDIRECTION, 
								// write value ignored.
	unsigned char pad;
    unsigned short taskHandle; // task number (offset in task array + 1, where 0 is 1) or handle thus usable from remote or C API, 4096 to 65535, set to 0 for public reg.
    unsigned short slength;    // this is reserved for later use and possible string length if want unsigned char, 0  255 values, VARIANT_BYTE, future type
	unsigned int indexCol;	  // Col dimension index reference, 0xffffffff for using default of current variant
	unsigned int indexRow;	  // Row dimension index reference
	union						// Data that was read or has been written of 'type'
	{
		int iValue;
		unsigned int uiValue;
		float fValue;
		double dValue;
		unsigned int dSwap[2];	// used to swap doubles for PC access
		char *psValue;
		char sValue[VARIANT_MAX_STRING+1];
								// will be stored in same VARIANT_STORAGE upon return, thus data.blockread.numAccess * sizeof(variant type)
		// if BLOCKACCESS then iValue[n], fValue[n], or dValue[n]
		struct 
		{
			BLOCKACCESS blockaccess;	// Defines block read of variant cells, data storage must be big enough since
			union	  
			{
				int ibValue[MAX_VARIANT_BLOCK_32BITS_SERIAL];	   
				float fbValue[MAX_VARIANT_BLOCK_32BITS_SERIAL];   
				double dbValue[MAX_VARIANT_BLOCK_64BITS_SERIAL];  
				union
				{
					int ibValue;	   
					float fbValue;   
					double dbValue;  
				} random[MAX_VARIANT_RANDOM_BLOCK_SERIAL];
				VARIANT_ACCESS_REQUEST request[MAX_VARIANT_RANDOM_BLOCK_SERIAL];
			};
		} block;
	} data;
} VARIANT_STORAGE_BLOCK_SERIAL;


// User DLL function interface for Variant Read/Write
typedef struct
{
	long lRegister;	// Desired register
	long type;		// Type of storage being used or requested
	long precision;	// Double to string conversion precision %.6f default
					// On read is what is presently set, write what want. 
	long flags;		// Special flags for processing so far only
					// VARIANT_INDIRECTION_FLAG used, can be used to set property
					// in ->settings on write operation, no effect on read.  Written
					// value becomes register to reference for further operations.
	long cmd;		// 00, no operation other than read/write specified, else do defined
					// operation.  Currently have write for properties access to 'settings'
					// VARIANT_CMD_SET_INDIRECTION and VARIANT_CMD_CLEAR_INDIRECTION, 
					// write value ignored.
	long taskHandle;  // task number or handle, 0 for public registers
	long slength;    // string length
	long indexCol;	// Column dimension index reference, unsigned short on controller
	long indexRow;	// Row dimension index reference, unsigned short on controller
	// Results type requested returned in relevant storage below:
	long LongVar;	
	float FloatVar;
	double DoubleVar;
	unsigned char StringArray[VARIANT_MAX_STRING+1];
} DLL_VARIANT_STORAGE;

// User DLL function interface for Variant Block Read


// User DLL function interface for Variant Block Read
typedef struct
{
	long lRegister;	// Desired register
	long type;		// Type of storage being used or requested
	long precision;	// Double to string conversion precision %.6f default
					// On read is what is presently set, write what want. 
	long flags;		// Special flags for processing so far only
					// VARIANT_INDIRECTION_FLAG used, can be used to set property
					// in ->settings on write operation, no effect on read.  Written
					// value becomes register to reference for further operations.
	long cmd;		// 00, no operation other than read/write specified, else do defined
					// operation.  Currently have write for properties access to 'settings'
					// VARIANT_CMD_SET_INDIRECTION and VARIANT_CMD_CLEAR_INDIRECTION, 
					// write value ignored.
	long taskHandle;  // task number or handle, 0 for public registers
	long slength;    // string length
	long indexCol;	// Column dimension index reference, unsigned short on controller
	long indexRow;	// Row dimension index reference, unsigned short on controller
	long numAccess; // Number of items to read
	long rowInc;	// row incrementor
	long colInc;	// column incrementor
	long arraysizeCols;	// Used on write operation, -1 do not expand existing columns, else columns desired.  Rows will automatically grow as needed
	// Results type requested returned in relevant storage below:
	union	  
	{
		int ibValue[MAX_VARIANT_BLOCK_32BITS];	   
		float fbValue[MAX_VARIANT_BLOCK_32BITS];   
		double dbValue[MAX_VARIANT_BLOCK_64BITS];  
		union
		{
			int ibValue;	   
			float fbValue;   
			double dbValue;  
		} random[MAX_VARIANT_RANDOM_BLOCK];
		VARIANT_ACCESS_REQUEST request[MAX_VARIANT_RANDOM_BLOCK];
	} block;
} DLL_VARIANT_STORAGE_BLOCK;

typedef struct
{
	long lRegister;	// Desired register
	long type;		// Type of storage being used or requested
	long precision;	// Double to string conversion precision %.6f default
					// On read is what is presently set, write what want. 
	long flags;		// Special flags for processing so far only
					// VARIANT_INDIRECTION_FLAG used, can be used to set property
					// in ->settings on write operation, no effect on read.  Written
					// value becomes register to reference for further operations.
	long cmd;		// 00, no operation other than read/write specified, else do defined
					// operation.  Currently have write for properties access to 'settings'
					// VARIANT_CMD_SET_INDIRECTION and VARIANT_CMD_CLEAR_INDIRECTION, 
					// write value ignored.
	long taskHandle;  // task number or handle, 0 for public registers
	long slength;    // string length
	long indexCol;	// Column dimension index reference, unsigned short on controller
	long indexRow;	// Row dimension index reference, unsigned short on controller
	long numAccess; // Number of items to read
	long rowInc;	// row incrementor
	long colInc;	// column incrementor
	long arraysizeCols;	// Used on write operation, -1 do not expand existing columns, else columns desired.  Rows will automatically grow as needed
	// Results type requested returned in relevant storage below:
	int ibValue[MAX_VARIANT_BLOCK_32BITS];	   
} DLL_VARIANT_STORAGE_BLOCK_INTEGER;

typedef struct
{
	long lRegister;	// Desired register
	long type;		// Type of storage being used or requested
	long precision;	// Double to string conversion precision %.6f default
					// On read is what is presently set, write what want. 
	long flags;		// Special flags for processing so far only
					// VARIANT_INDIRECTION_FLAG used, can be used to set property
					// in ->settings on write operation, no effect on read.  Written
					// value becomes register to reference for further operations.
	long cmd;		// 00, no operation other than read/write specified, else do defined
					// operation.  Currently have write for properties access to 'settings'
					// VARIANT_CMD_SET_INDIRECTION and VARIANT_CMD_CLEAR_INDIRECTION, 
					// write value ignored.
	long taskHandle;  // task number or handle, 0 for public registers
	long slength;    // string length
	long indexCol;	// Column dimension index reference, unsigned short on controller
	long indexRow;	// Row dimension index reference, unsigned short on controller
	long numAccess; // Number of items to read
	long rowInc;	// row incrementor
	long colInc;	// column incrementor
	long arraysizeCols;	// Used on write operation, -1 do not expand existing columns, else columns desired.  Rows will automatically grow as needed
	// Results type requested returned in relevant storage below:
	int fbValue[MAX_VARIANT_BLOCK_32BITS];	   
} DLL_VARIANT_STORAGE_BLOCK_FLOAT;

typedef struct
{
	long lRegister;	// Desired register
	long type;		// Type of storage being used or requested
	long precision;	// Double to string conversion precision %.6f default
					// On read is what is presently set, write what want. 
	long flags;		// Special flags for processing so far only
					// VARIANT_INDIRECTION_FLAG used, can be used to set property
					// in ->settings on write operation, no effect on read.  Written
					// value becomes register to reference for further operations.
	long cmd;		// 00, no operation other than read/write specified, else do defined
					// operation.  Currently have write for properties access to 'settings'
					// VARIANT_CMD_SET_INDIRECTION and VARIANT_CMD_CLEAR_INDIRECTION, 
					// write value ignored.
	long taskHandle;  // task number or handle, 0 for public registers
	long slength;    // string length
	long indexCol;	// Column dimension index reference, unsigned short on controller
	long indexRow;	// Row dimension index reference, unsigned short on controller
	long numAccess; // Number of items to read
	long rowInc;	// row incrementor
	long colInc;	// column incrementor
	long arraysizeCols;	// Used on write operation, -1 do not expand existing columns, else columns desired.  Rows will automatically grow as needed
	// Results type requested returned in relevant storage below:
	double dbValue[MAX_VARIANT_BLOCK_64BITS];  
} DLL_VARIANT_STORAGE_BLOCK_DOUBLE;

typedef struct
{
	long lRegister;	// Desired register
	long type;		// Type of storage being used or requested
	long precision;	// Double to string conversion precision %.6f default
					// On read is what is presently set, write what want. 
	long flags;		// Special flags for processing so far only
					// VARIANT_INDIRECTION_FLAG used, can be used to set property
					// in ->settings on write operation, no effect on read.  Written
					// value becomes register to reference for further operations.
	long cmd;		// 00, no operation other than read/write specified, else do defined
					// operation.  Currently have write for properties access to 'settings'
					// VARIANT_CMD_SET_INDIRECTION and VARIANT_CMD_CLEAR_INDIRECTION, 
					// write value ignored.
	long taskHandle;  // task number or handle, 0 for public registers
	long slength;    // string length
	long indexCol;	// Column dimension index reference, unsigned short on controller
	long indexRow;	// Row dimension index reference, unsigned short on controller
	long numAccess; // Number of items to read
	long rowInc;	// row incrementor
	long colInc;	// column incrementor
	long arraysizeCols;	// Used on write operation, -1 do not expand existing columns, else columns desired.  Rows will automatically grow as needed
	// Results type requested returned in relevant storage below:
	double dbValue[MAX_VARIANT_RANDOM_BLOCK];
	VARIANT_ACCESS_REQUEST_VB request[MAX_VARIANT_RANDOM_BLOCK];
} DLL_VARIANT_STORAGE_BLOCK_DOUBLE_RANDOM;



// function prototypes (public)
// network access functions
DLLEXPORT ULONG WINAPI CtOpenNetConnection( ULONG, ULONG, ULONG, ULONG );
DLLEXPORT ULONG WINAPI CtOpenConnection( ULONG, ULONG, ULONG );
DLLEXPORT ULONG WINAPI CtInitConnection( ULONG, DWORD, BYTE, BYTE, BYTE );
DLLEXPORT ULONG WINAPI CtCloseConnection( ULONG );
DLLEXPORT ULONG WINAPI CtSetConnectionTimeout( ULONG, ULONG );
DLLEXPORT ULONG WINAPI CtSetAdapter(ULONG id);
DLLEXPORT ULONG WINAPI CtGetAvailableAdaptersIPList(ULONG FAR *ipList, ULONG FAR *ipAdapter, ULONG FAR *numEntries);
DLLEXPORT ULONG WINAPI CtUnload(void);


// register functions
DLLEXPORT ULONG WINAPI CtGetRegister( ULONG, ULONG, LPLONG );
DLLEXPORT ULONG WINAPI CtGetRegister16( ULONG, ULONG, REGS_16 FAR* );
DLLEXPORT ULONG WINAPI CtGetRegister50( ULONG, ULONG, REGS_50 FAR* );
DLLEXPORT ULONG WINAPI CtPutRegister( ULONG, ULONG, LONG );

// flag functions
DLLEXPORT ULONG WINAPI CtGetFlag( ULONG, ULONG, LPLONG );
DLLEXPORT ULONG WINAPI CtPutFlag( ULONG, ULONG, ULONG );

// digital io functions
DLLEXPORT ULONG WINAPI CtGetDigitalInput8( ULONG, ULONG, LPLONG );
DLLEXPORT ULONG WINAPI CtGetDigitalOutput8( ULONG, ULONG, LPLONG );
DLLEXPORT ULONG WINAPI CtGetDigitalInput128( ULONG, ULONG, DIGS_128 FAR* );
DLLEXPORT ULONG WINAPI CtGetDigitalOutput128( ULONG, ULONG, DIGS_128 FAR* );

// analog io functions
DLLEXPORT ULONG WINAPI CtGetAnalogInput( ULONG, ULONG, LPLONG );
DLLEXPORT ULONG WINAPI CtGetAnalogOutput( ULONG, ULONG, LPLONG );
DLLEXPORT ULONG WINAPI CtPutAnalogOutput( ULONG, ULONG, ULONG );
DLLEXPORT ULONG WINAPI CtGetAnalogInput32( ULONG, ULONG, ANGS_32 FAR* );
DLLEXPORT ULONG WINAPI CtGetAnalogOutput32( ULONG, ULONG, ANGS_32 FAR* );

// servo functions
DLLEXPORT ULONG WINAPI CtGetServoPosition( ULONG, ULONG, LPLONG );
DLLEXPORT ULONG WINAPI CtGetServoError( ULONG, ULONG, LPLONG );
DLLEXPORT ULONG WINAPI CtGetServoInputs( ULONG, ULONG, LPLONG );

// data table functions
DLLEXPORT ULONG WINAPI CtGetDataTableDim( ULONG, LPDWORD, LPDWORD );
DLLEXPORT ULONG WINAPI CtPutDataTableDim( ULONG, ULONG, ULONG );
DLLEXPORT ULONG WINAPI CtGetDataTableLoc( ULONG, ULONG, ULONG, LPLONG );
DLLEXPORT ULONG WINAPI CtPutDataTableLoc( ULONG, ULONG, ULONG, ULONG );
DLLEXPORT ULONG WINAPI CtGetDataTableRow( ULONG, ULONG, ULONG, ULONG, CT_DT_ROW FAR* ) ;
DLLEXPORT ULONG WINAPI CtPutDataTableRow( ULONG, ULONG, ULONG, ULONG, CT_DT_ROW FAR* ) ;

// controller status functions
DLLEXPORT ULONG WINAPI CtGetStatus( ULONG, LPLONG );
DLLEXPORT ULONG WINAPI CtPutStatus( ULONG, ULONG );
DLLEXPORT ULONG WINAPI CtGetTaskBank( ULONG, ULONG, CT_TASK_BANK FAR* );

// controller configuration functions
DLLEXPORT ULONG WINAPI CtGetModel( ULONG, LPLONG, BOOL FAR* );
DLLEXPORT ULONG WINAPI CtGetConfig( ULONG, LPLONG );
DLLEXPORT ULONG WINAPI CtPutConfig( ULONG, ULONG );
DLLEXPORT ULONG WINAPI CtGetIoCount( ULONG, CT_IO_COUNT FAR* );
DLLEXPORT ULONG WINAPI CtGetMiscIoCount( ULONG, CT_MISC_IO_COUNT FAR* );

// programming functions
DLLEXPORT ULONG WINAPI CtDownload( ULONG, const char FAR*, CT_PROG_COUNT FAR*);
DLLEXPORT ULONG WINAPI CtUpload( ULONG, const char FAR*, CT_PROG_COUNT FAR*);
DLLEXPORT ULONG WINAPI CtUncompile ( LONG model,
									const char FAR *objname, 
									const char FAR *srcname, 
									const char FAR *tabname, 
									const char FAR *figname, 
									CT_PROG_COUNT FAR *pCounts);

// miscellaneous functions
DLLEXPORT ULONG WINAPI CtGetConnectionInfo( ULONG, CT_CONN_INFO FAR* );
DLLEXPORT ULONG WINAPI CtGetMessageInfo( ULONG, CT_MSG_INFO FAR* );
DLLEXPORT ULONG WINAPI CtGetErrorInfo( ULONG, CT_ERR_INFO FAR* );
DLLEXPORT ULONG WINAPI CtSendData( ULONG, USHORT, BYTE FAR*, USHORT, BYTE FAR* );
DLLEXPORT ULONG WINAPI CtSendAscii( ULONG, BYTE FAR*, USHORT, BYTE FAR*, BYTE );

// message structure for 2217 data
typedef struct {
	LONG	lAnalIn[128];
	LONG	lAnalOut[128];
	LONG	lDigIn[10];
	LONG	lDigOut[10];
	LONG	lReg[100];
} CT_2217DATA;
// special 2217 v3.8 message for collected IO data
DLLEXPORT ULONG WINAPI CtGet2217Data( ULONG, CT_2217DATA FAR* );

// devicenet and/or distributed io messages
DLLEXPORT ULONG WINAPI CtLoadRemoteIOConfig( ULONG, CT_2217DATA FAR* );
DLLEXPORT ULONG WINAPI CtUnloadRemoteIOConfig( ULONG, CT_2217DATA FAR* );
DLLEXPORT ULONG WINAPI CtGetRemoteIOData( ULONG, CT_2217DATA FAR* );


// Variant access
DLLEXPORT ULONG WINAPI CtGetVProperties(ULONG lConnectID, ULONG lIndex, ULONG *pSizeX_Column, ULONG *pSizeY_Row, ULONG *pPrecision);
DLLEXPORT ULONG WINAPI CtGetVRegister(ULONG lConnectID, DLL_VARIANT_STORAGE *variant);
DLLEXPORT ULONG CtGetVRegisterRandomBlock(ULONG lConnectID, DLL_VARIANT_STORAGE_BLOCK *variant);
ULONG _stdcall CtGetVRegisterRandomBlockDouble(ULONG lConnectID, DLL_VARIANT_STORAGE_BLOCK_DOUBLE_RANDOM *variant);
DLLEXPORT ULONG WINAPI CtGetVRegisterBlock(ULONG lConnectID, DLL_VARIANT_STORAGE_BLOCK *variant);
DLLEXPORT ULONG WINAPI CtGetVRegisterBlockInteger(ULONG lConnectID, DLL_VARIANT_STORAGE_BLOCK_INTEGER *variant);
DLLEXPORT ULONG WINAPI CtSetVRegisterBlockInteger(ULONG lConnectID, DLL_VARIANT_STORAGE_BLOCK_INTEGER *variant);
DLLEXPORT ULONG WINAPI CtSetVRegisterBlockFloat(ULONG lConnectID, DLL_VARIANT_STORAGE_BLOCK_FLOAT *variant);
DLLEXPORT ULONG WINAPI CtSetVRegisterBlockDouble(ULONG lConnectID, DLL_VARIANT_STORAGE_BLOCK_DOUBLE *variant);
DLLEXPORT ULONG WINAPI CtSetVRegisterBlock(ULONG lConnectID, DLL_VARIANT_STORAGE_BLOCK *variant);
DLLEXPORT ULONG WINAPI CtGetVRegisterBlockFloat(ULONG lConnectID, DLL_VARIANT_STORAGE_BLOCK_FLOAT *variant);
DLLEXPORT ULONG WINAPI CtGetVRegisterBlockDouble(ULONG lConnectID, DLL_VARIANT_STORAGE_BLOCK_DOUBLE *variant);
DLLEXPORT ULONG WINAPI CtGetVRegisters(ULONG lConnectID, ULONG numEntries, ULONG *doneEntries, DLL_VARIANT_STORAGE *variant);
DLLEXPORT ULONG WINAPI CtSetVRegister(ULONG lConnectID,  DLL_VARIANT_STORAGE *variant);
DLLEXPORT ULONG WINAPI CtSetVRegisters(ULONG lConnectID,  ULONG numEntries, ULONG *doneEntries, DLL_VARIANT_STORAGE *variant);
DLLEXPORT ULONG WINAPI CtRunCommand(ULONG lConnectID, BYTE *cmd, ULONG cmdLength, BYTE *result, ULONG *resultLength);
DLLEXPORT ULONG WINAPI CtGetNRegisters(ULONG lConnectID, ULONG lRegisterCount, ULONG* pRegisterList, LPLONG pValues);




#endif // INC_CTCCOM32_H
